from nose import tools
from six import moves
import mock

import phlawg

LEVELS = [
   'critical',
   'debug',
   'error',
   'fatal',
   'info',
   'warn',
   'warning',
]

class TestPhlawgMetricLogger(object):
    def setup(self):
        self.base_logger = mock.Mock(name='BaseLogger')
        self.logger = phlawg.MetricLogger(self.base_logger)

    def metrics(self, count=3):
        # Produces a dictionary of metrics as key/value pairs, with each
        # key having a dynamic name (by stringifying a mock) and a dynamic
        # integer value (the id of a new mock).
        # The dictionary will have `count` pairs.
        return dict((str(mock.Mock(name='metricName')), id(mock.Mock()))
                    for i in moves.xrange(count))

    def check_metrics(self, level, **metrics):
        pass

    def check_unordered_calls(self, calls, expectation):
        tools.assert_equal(
                sorted(calls),
                sorted(expectation))


    def test_log_method(self):
        level = mock.Mock(name='LogLevel')
        metrics = self.metrics()
        self.logger.log(level, **metrics)

        self.check_unordered_calls(
            self.base_logger.log.call_args_list,
            [((level, '%s=%s' % (name, val)), {'extra': {'metric': name, 'value': val}})
             for name, val in metrics.items()])

    def test_level_methods(self):
        for level in LEVELS:
            yield self.verify_level_method, level

    def verify_level_method(self, level):
        emitter = getattr(self.logger, level)
        receiver = getattr(self.base_logger, level)
        metrics = self.metrics()
        emitter(**metrics)
        self.check_unordered_calls(
            receiver.call_args_list,
            [(('%s=%s' % (name, val),), {'extra': {'metric': name, 'value': val}})
             for name, val in metrics.items()])




"""
Functions for configuring the logging system in phlawg-friendly ways.
"""

from __future__ import absolute_import

import json
import os
from logging import config as logconf

import six

import phlawg

METRIC_HANDLER_KEY = 'phlawg_metrics_handler'
METRIC_FORMATTER_KEY = 'phlawg_metrics_formatter'
LOG_HANDLER_KEY = 'phlawg_default_handler'
LOG_FORMATTER_KEY = 'phlawg_default_formatter'

DEFAULT_METRIC_FIELDS = (
    'asctime', 'name', 'levelname', 'process', 'thread', 'message')

def metric_logger_specification(name):
    return {"qualname": name,
            "level": "INFO",
            "propagate": False,
            "handlers": [METRIC_HANDLER_KEY],
            }

def metric_formatter_specification():
    return {'()': 'pythonjsonlogger.jsonlogger.JsonFormatter',
            'format': metric_field_format(DEFAULT_METRIC_FIELDS),
            }

def metric_handler_specification():
    return {'formatter': 'phlawg_metrics_formatter',
            'class': 'logging.StreamHandler',
            'stream': 'ext://sys.stderr',
            }

def metric_field_format(fields):
    return ' '.join('(%s)' % fld for fld in fields)


def default_config():
    return {
        'loggers': {},
        'root': {
            'level': 'INFO',
            'handlers': ['phlawg_default_handler'],
            },
        'formatters': {
            'phlawg_metrics_formatter': metric_formatter_specification(),
            'phlawg_default_formatter': {
                'format': '%(asctime)s %(levelname)s #%(process)d %(thread)d %(name)s %(message)s'
                },
            },
        'handlers': {
            'phlawg_metrics_handler': metric_handler_specification(),
            'phlawg_default_handler': {
                'formatter': 'phlawg_default_formatter',
                'class': 'logging.StreamHandler',
                'stream': 'ext://sys.stderr',
                },
            },
        }

def env_list(variable):
    val = os.getenv(variable)
    return val.split(',') if val else []


def env_var(variable, default=None, handler=lambda x: x):
    val = os.getenv(variable)
    if val is not None and val != '':
        return handler(val)
    return default


class EnvConf(object):
    METRIC_PACKAGES_VAR = 'PHLAWG_METRIC_PACKAGES'
    METRIC_FIELDS_VAR = 'PHLAWG_METRIC_FIELDS'
    METRIC_DATE_FORMAT_VAR = 'PHLAWG_METRIC_DATE_FORMAT'
    METRIC_LEVEL_VAR = 'PHLAWG_METRIC_LEVEL'
    LOG_FORMAT_VAR = 'PHLAWG_LOG_FORMAT'
    DATE_FORMAT_VAR = 'PHLAWG_LOG_DATE_FORMAT'
    LOG_LEVEL_VAR = 'PHLAWG_LOG_LEVEL'
    FULL_CONF_VAR = 'PHLAWG_LOG_CONFIG'

    def __init__(self, metric_packages=()):
        self.metric_packages = self.determine_metric_packages(*metric_packages)
        self.metric_fields = self.determine_metric_fields()
        self.metric_level = self.determine_metric_level()
        self.log_format = self.determine_log_format()
        self.log_date_format = self.determine_log_date_format()
        self.log_level = self.determine_log_level()
        self.metric_date_format = self.determine_metric_date_format()
        self.specification = self.determine_specification()


    @classmethod
    def determine_metric_packages(cls, *app_metric_packages):
        app_metric_packages += tuple(env_list(cls.METRIC_PACKAGES_VAR))
        if app_metric_packages:
            return [phlawg.to_metric_logger_name(name)
                    for name in app_metric_packages]
        return ['metrics']

    @classmethod
    def determine_specification(cls):
        val = env_var(cls.FULL_CONF_VAR, default=None, handler=json.loads)
        if val:
            cls.ensure_metric_handler(val)
            cls.ensure_metric_formatter(val)
        return val


    @classmethod
    def ensure_metric_handler(cls, conf):
        if METRIC_HANDLER_KEY not in conf["handlers"]:
            conf["handlers"][METRIC_HANDLER_KEY] = \
                    metric_handler_specification()

    @classmethod
    def ensure_metric_formatter(cls, conf):
        if METRIC_FORMATTER_KEY not in conf["formatters"]:
            conf["formatters"][METRIC_FORMATTER_KEY] = \
                    metric_formatter_specification()

    @classmethod
    def determine_metric_fields(cls):
        return env_list(cls.METRIC_FIELDS_VAR)

    @classmethod
    def determine_metric_level(cls):
        return env_var(cls.METRIC_LEVEL_VAR)

    @classmethod
    def determine_log_level(cls):
        return env_var(cls.LOG_LEVEL_VAR)

    @classmethod
    def determine_log_format(cls):
        return env_var(cls.LOG_FORMAT_VAR)

    @classmethod
    def determine_log_date_format(cls):
        return env_var(cls.DATE_FORMAT_VAR)

    @classmethod
    def determine_metric_date_format(cls):
        return env_var(cls.METRIC_DATE_FORMAT_VAR)

    def apply_metric_loggers(self, conf):
        for name in self.metric_packages:
            if name not in conf["loggers"]:
                conf["loggers"][name] = metric_logger_specification(name)

    def apply_metric_fields(self, conf):
        if self.metric_fields:
            conf["formatters"][METRIC_FORMATTER_KEY]['format'] = (
                    metric_field_format(self.metric_fields))

    def apply_metric_level(self, conf):
        if self.metric_level:
            conf["handlers"][METRIC_HANDLER_KEY]['level'] = self.metric_level

    def apply_log_level(self, conf):
        if self.log_level:
            conf["handlers"][LOG_HANDLER_KEY]['level'] = self.log_level

    def apply_log_format(self, conf):
        if self.log_format:
            conf["formatters"][LOG_FORMATTER_KEY]['format'] = self.log_format

    def apply_log_date_format(self, conf):
        if self.log_date_format:
            conf["formatters"][LOG_FORMATTER_KEY]['datefmt'] = \
                    self.log_date_format

    def apply_metric_date_format(self, conf):
        if self.metric_date_format:
            conf["formatters"][METRIC_FORMATTER_KEY]['datefmt'] = \
                    self.metric_date_format


    @property
    def config(self):
        conf = self.specification if self.specification else default_config()
        self.apply_metric_loggers(conf)
        if not self.specification:
            self.apply_metric_fields(conf)
            self.apply_metric_level(conf)
            self.apply_log_level(conf)
            self.apply_log_format(conf)
            self.apply_log_date_format(conf)
            self.apply_metric_date_format(conf)
        conf['version'] = 1
        return conf


def from_environment(*metric_packages):
    """
    Configures python's logging system based on environment variables.

    Supported environment variables:
        ``PHLAWG_METRIC_FIELDS``: Comma-separated list of log record field names
            to include in the JSON metric output.

        ``PHLAWG_METRIC_PACKAGES``: Python package/subpackage/module names under
            which metrics are expected to be logged.  Loggers with suitable
            qualnames, using the "phlawg_metric_handler" handler, will be added
            to the configuration.  Names will be converted to metric logger names
            based on :func:`phlawg.to_metric_logger_name`.

        ``PHLAWG_METRIC_DATE_FORMAT``: The date formatting string to use for the
            time field within metric logs.

        ``PHLAWG_METRIC_LEVEL``: The logging level name to use for metric logs.

        ``PHLAWG_LOG_FORMAT``: The log format string to use for regular log lines.

        ``PHLAWG_LOG_DATE_FORMAT``: The date formatting string to use for the
            time field within regular logs.

        ``PHLAWG_LOG_LEVEL``: The logging level name to use for regular logs.

        ``PHLAWG_LOG_CONFIG``: a full log configuration dictionary as would be
            passed to :func:`logging.config.dictConfig`, encoded as JSON.  Note
            that this will be subject to some modification; the
            "phlawg_metric_handler" and "phlawg_metric_formatter" structures will
            be ensured, with default definitions if not already present.  Any
            metric logger names determined via metric package name resolution
            will be ensured as well within the "loggers" dict.

    Defaults are provided for everything such that no environmental configuration
    is strictly required in order to use this and get a sensible behavior.

    Metric package names can be expressed both in the ``PHLAWG_METRIC_PACKAGES``
    environment variable and in the call to this function as positional parameters;
    these will be merged, and all subject to :func:`phlawg.to_metric_logger_name`
    in determining the logger/qualname.

    Metric loggers will use the :class:`pythonjsonlogger.jsonlogger.JsonFormatter`
    formatter to express themselves as JSON dictionaries in the logstream.

    Logging levels are applied to the log handlers, not the loggers themselves.

    All logs will go to STDERR by default.

    Returns ``True``.
    """
    logconf.dictConfig(EnvConf(metric_packages).config)
    return True




import os
import json
from nose import tools
import mock
import six

from phlawg import config

FULL_CONF_VAR = 'PHLAWG_LOG_CONFIG'
LOG_FORMAT_VAR = 'PHLAWG_LOG_FORMAT'
DATE_FORMAT_VAR = 'PHLAWG_LOG_DATE_FORMAT'
METRIC_FIELDS_VAR = 'PHLAWG_METRIC_FIELDS'
METRIC_PACKAGES_VAR = 'PHLAWG_METRIC_PACKAGES'
METRIC_DATE_FORMAT_VAR = 'PHLAWG_METRIC_DATE_FORMAT'
LOG_LEVEL_VAR = 'PHLAWG_LOG_LEVEL'
METRIC_LEVEL_VAR = 'PHLAWG_METRIC_LEVEL'

ALL_VARS = [
        FULL_CONF_VAR, LOG_FORMAT_VAR, DATE_FORMAT_VAR,
        METRIC_FIELDS_VAR, METRIC_PACKAGES_VAR]

DEFAULT_FORMAT = '%(asctime)s %(levelname)s #%(process)d %(thread)d %(name)s ' \
        '%(message)s'

# The pythonjsonlogger.jsonlogger.JsonFormatter identifies the
# required JSON fields from the format string, but doesn't apply
# formatting specifications (the formatting comes from the JSONification).
# So we just identify fields of interest.
METRIC_FORMAT = '(asctime) (name) (levelname) (process) (thread) (message)'

# The default config specs for the various portions of the
# logging.  We have a "metric" and a "default" path through
# the logs, each with their own handlers.  We have a "phlawg_"
# prefix so we can isolate things.
def default_log_spec(handlers=['phlawg_default_handler'],
        level='INFO', **kw):
    return dict(kw, handlers=handlers, level=level)

def metric_log_spec(handlers=['phlawg_metrics_handler'], propagate=False,
        qualname='metrics', **kw):
    return default_log_spec(
        **dict(kw, handlers=handlers, propagate=propagate, qualname=qualname))

def default_handler_spec(formatter='phlawg_default_formatter',
        stream='ext://sys.stderr', **kw):
    if 'class' not in kw:
        kw['class'] = 'logging.StreamHandler'
    return dict(kw, formatter=formatter, stream=stream)

def metric_handler_spec(formatter='phlawg_metrics_formatter', **kw):
    return default_handler_spec(**dict(kw, formatter=formatter))

def default_formatter_spec(format=DEFAULT_FORMAT, datefmt=None, **kw):
    r = dict(kw, format=format, datefmt=datefmt)
    if not r['datefmt']:
        del r['datefmt']
    return r

def metric_formatter_spec(format=METRIC_FORMAT, **kw):
    if '()' not in kw:
        kw['()'] = 'pythonjsonlogger.jsonlogger.JsonFormatter'
    return default_formatter_spec(**dict(kw, format=format))

def default_config():
    return {
        "version": 1,
        "root": default_log_spec(),
        "loggers": {
            "metrics": metric_log_spec(),
            },
        "handlers": {
            "phlawg_default_handler": default_handler_spec(),
            "phlawg_metrics_handler": metric_handler_spec(),
            },
        "formatters": {
            "phlawg_default_formatter": default_formatter_spec(),
            "phlawg_metrics_formatter": metric_formatter_spec(),
            },
        }


def mocks(fn):
    @six.wraps(fn)
    def wrapped(*a, **kw):
        with mock.patch.dict(os.environ):
            with mock.patch('logging.config.dictConfig') as dictconf:
                for var in ALL_VARS:
                    if var in os.environ:
                        del os.environ[var]
                return fn(*(a + (os.environ, dictconf)), **kw)
    return wrapped


def add_metric_loggers(conf, *names):
    for name in names:
        conf['loggers'][name] = metric_log_spec(qualname=name)
    return conf


# This gives a more detailed comparison on failure than the
# usual assert_called_once_with() does.
def comparable_call(logconf, expect):
    tools.assert_equal(1, len(logconf.call_args_list))
    # Further verify the shape of the call arguments.
    tools.assert_equal(
            (1, {}), (
                len(logconf.call_args_list[0][0]),
                logconf.call_args_list[0][1]))
    # Now directly compare the dicts.
    val = logconf.call_args_list[0][0][0]
    # Verify top-level keys
    tools.assert_equal(
            sorted(expect.keys()), sorted(val.keys()))
    # Verify each value
    for k in sorted(expect.keys()):
        tools.assert_equal(expect[k], val[k])


@mocks
def test_default_config(env, logconf):
    # With nothing in the environment, we should get the default specification.
    config.from_environment()
    comparable_call(logconf, default_config())

@mocks
def test_log_level(env, logconf):
    env[LOG_LEVEL_VAR] = str(mock.Mock(name='SomeLogLevelName'))
    config.from_environment()
    expect = default_config()
    expect["handlers"]["phlawg_default_handler"]["level"] = env[LOG_LEVEL_VAR]
    comparable_call(logconf, expect)

@mocks
def test_metric_level(env, logconf):
    env[METRIC_LEVEL_VAR] = str(mock.Mock(name='SomeLogLevelName'))
    config.from_environment()
    expect = default_config()
    expect["handlers"]["phlawg_metrics_handler"]["level"] = \
            env[METRIC_LEVEL_VAR]
    comparable_call(logconf, expect)

@mocks
def test_default_config_with_app_metric_names(env, logconf):
    # Default config (nothing in environment), but the app specifies
    # additional metric packages, so we expect additional metric-handled
    # loggers to result.
    config.from_environment('topguy', 'second.guy')
    expect = default_config()
    add_metric_loggers(expect, 'topguy.metrics', 'second.metrics.guy')
    # But in specifying packages, we throw away the default one
    del expect['loggers']['metrics']
    comparable_call(logconf, expect)


@mocks
def test_default_config_with_env_metric_names(env, logconf):
    # comma-separated list of packages with metrics.
    env[METRIC_PACKAGES_VAR] = 'envguy,other.guy'
    expect = default_config()
    add_metric_loggers(expect, 'envguy.metrics', 'other.metrics.guy')
    # Throw out the default, as before.
    del expect['loggers']['metrics']
    config.from_environment()
    comparable_call(logconf, expect)


@mocks
def test_default_config_with_both_metric_names(env, logconf):
    # Some packages from the environment
    env[METRIC_PACKAGES_VAR] = 'envguy,env.other,common'
    expect = default_config()
    add_metric_loggers(expect,
            'envguy.metrics', 'env.metrics.other', 'common.metrics',
            'appguy.metrics', 'app.metrics.other')
    # Again, no default
    del expect['loggers']['metrics']
    # The app specifies two names.
    config.from_environment('appguy', 'app.other')
    comparable_call(logconf, expect)


@mocks
def test_metric_fields_var(env, logconf):
    fields = ['some', 'other', 'fields']
    # We express the desired fields in a comma-separated list.
    env[METRIC_FIELDS_VAR] = ','.join(fields)
    expect = default_config()
    # And in the log formatter, the JsonFormatter just needs to see
    # the field names in parentheses.
    expect['formatters']['phlawg_metrics_formatter']['format'] = \
            ' '.join('(%s)' % fld for fld in fields)
    config.from_environment()
    comparable_call(logconf, expect)


@mocks
def test_log_format_var(env, logconf):
    format_str = str(mock.Mock(name='AFakeLogFormatString'))
    env[LOG_FORMAT_VAR] = format_str
    expect = default_config()
    expect['formatters']['phlawg_default_formatter']['format'] = format_str
    config.from_environment()
    comparable_call(logconf, expect)


@mocks
def test_date_format_var(env, logconf):
    fmt = str(mock.Mock(name='AFakeDateFormatString'))
    env[DATE_FORMAT_VAR] = fmt
    expect = default_config()
    expect['formatters']['phlawg_default_formatter']['datefmt'] = fmt
    config.from_environment()
    comparable_call(logconf, expect)


@mocks
def test_metric_date_format_var(env, logconf):
    fmt = str(mock.Mock(name='AnotherFakeDateFormatString'))
    env[METRIC_DATE_FORMAT_VAR] = fmt
    expect = default_config()
    expect['formatters']['phlawg_metrics_formatter']['datefmt'] = fmt
    config.from_environment()
    comparable_call(logconf, expect)


def mock_dict():
    return {
            str(mock.Mock(name='AKey')): str(mock.Mock(name='AValue'))}


def override_case(spec):
    def wrapper(fn):
        @six.wraps(fn)
        @mocks
        def wrapped(env, logconf):
            # Put the specification into the variable as JSON.
            env[FULL_CONF_VAR] = json.dumps(spec)
            # Build expectation from JSON (so we get a copy).
            expect = json.loads(env[FULL_CONF_VAR])
            expect, packages = fn(expect, env, logconf)
            # Apply the configuration and verify our expectation.
            config.from_environment(*packages)
            comparable_call(logconf, expect)
        return wrapped
    return wrapper

@override_case({
    "loggers": {
        "metrics": mock_dict(),
        },
    "handlers": {
        "phlawg_metrics_handler": mock_dict(),
        },
    "formatters": {
        "phlawg_metrics_formatter": mock_dict(),
        },
    })
def test_override_full_definition(spec, env, logconf):
    # If all the metrics needed have loggers, and
    # metric handler and formatter are both present,
    # we accept the override JSON definition as-is.
    
    # We expect it to add a version for us.
    spec["version"] = 1
    return spec, ()


@override_case({
    "loggers": {
        # It'll accept this
        "fromenv.metrics.defined": mock_dict(),
        # And this.
        "fromapp.metrics.defined": mock_dict(),
        },
    "handlers": {
        "phlawg_metrics_handler": mock_dict(),
        },
    "formatters": {
        "phlawg_metrics_formatter": mock_dict(),
        },
    })
def test_override_with_metric_names(spec, env, logconf):
    # In this case, we check that it adds a metric logger
    # for one that was missing, but leaves the ones it
    # finds alone.

    # Env-level metrics packages requested.
    env[METRIC_PACKAGES_VAR] = 'fromenv.defined,fromenv.undefined'

    # These guys will be added by the system based on the metrics
    # packages we requested.
    for name in ("fromenv.metrics.undefined",
                 "fromapp.metrics.undefined"):
        spec["loggers"][name] = metric_log_spec(qualname=name)

    # And of course we expect version 1.
    spec["version"] = 1

    # App-level metrics packages requested
    return spec, ("fromapp.defined", "fromapp.undefined")


@override_case({
    "loggers": {
        "metrics": mock_dict(),
        },
    "handlers": {
        "phlawg_metrics_handler": mock_dict(),
        "other_metrics_handler": mock_dict(),
        },
    "formatters": {
        "some_formtter": mock_dict(),
        },
    })
def test_override_with_metric_handler(spec, env, logconf):
    # We should see that it accepts the given metrics handler,
    # but provides the standard metrics formatter.
    spec["formatters"]["phlawg_metrics_formatter"] = metric_formatter_spec()
    spec["version"] = 1
    return spec, ()


@override_case({
    "loggers": {
        "metrics": mock_dict(),
        },
    "handlers": {
        "other_handler": mock_dict(),
        },
    "formatters": {
        "phlawg_metrics_formatter": mock_dict(),
        "other_formatter": mock_dict(),
        },
    })
def test_override_with_metric_formatter(spec, env, logconf):
    # We should see that it accepts the given metric formatter,
    # but adds the standard metrics handler.
    spec["handlers"]["phlawg_metrics_handler"] = metric_handler_spec()
    spec["version"] = 1
    return spec, ()


@override_case({
    "loggers": {
        "some_logger": mock_dict(),
        "other_logger": mock_dict(),
        },
    "handlers": {
        "some_handler": mock_dict(),
        "other_handler": mock_dict(),
        },
    "formatters": {
        "some_formatter": mock_dict(),
        "other_formatter": mock_dict(),
        },
    })
def test_override_with_no_metrics(spec, env, logconf):
    # In this case, the override configuration doesn't deal with our metrics
    # structures at all, so we expect the system to provide for them.
    spec["loggers"]["metrics"] = metric_log_spec()
    spec["handlers"]["phlawg_metrics_handler"] = metric_handler_spec()
    spec["formatters"]["phlawg_metrics_formatter"] = metric_formatter_spec()
    spec["version"] = 1
    return spec, ()




import six

class MetricLogger(object):
    """A wrapper class for logging.Loggers for metric propagation through log streams.

    A MetricLogger may be used mostly like a traditional logging.Logger, as it has the
    same event emission methods as the logging.Logger it wraps (methods either corresponding
    to a logging level name or the generic `log` method).

    Metrics are expressed as keyword parameters, with a keyword being a simple name,
    and the corresponding value being the numeric metric value.

    Each metric results in its own log record at the specified level, transmitted through
    the wrapped logger, with the metric being expressed both as a simple "key=value" string
    in the log message, and with the "extra" dictionary having 'metric' and 'value' members.
    Consequently, every metric can be extracted from the log lines provided those log lines
    capture either the "extra" dictionary structure or the message field.

    While the MetricLogger does not enforce this convention, the intended use is for the
    metric keys to be thought of as simple names within an overall namespace, determined
    by the name of the logger itself.  So, I might do something like this:

            logger = phlawg.MetricLogger(logging.getLogger('foo.metrics.bar'))
            logger.info(metric_a=0, metric_b=-6.5)

    And I would logically think of the metrics as "foo.metrics.bar.metric_a" and
    "foo.metrics.bar.metric_b".

    Extend MetricLogger and override `message` and or `extra` to control how the log lines
    are formatted.
    """

    def __init__(self, logger):
        """Wrap a logging.Logger-like `logger` with metrics-emitting behaviors."""
        self.logger = logger

    def message_and_extra(self, metrics):
        """For each metric/value pair in `metrics`, yields the log message and 'extra' dictionary."""
        for name, value in six.iteritems(metrics):
            yield self.message(name, value), self.extra(name, value)

    def message(self, name, value):
        """Formats and returns a log message string for the metric name,value pair"""
        return "%s=%s" % (name, value)

    def extra(self, name, value):
        """Returns a dictionary suitable for use as the log's `extra` keyword parameter for the
        name,value metric pair."""
        return {'metric': name, 'value': value}


    def log(self, level, **metrics):
        """Log the metrics expressed in keyword arguments using the specified log `level`."""
        for msg, xtra in self.message_and_extra(metrics):
            self.logger.log(level, msg, extra=xtra)


    def _level_emit(self, emitter, metrics):
        for msg, xtra in self.message_and_extra(metrics):
            emitter(msg, extra=xtra)

    def critical(self, **metrics):
        """Log the metrics expressed in keyword arguments at logging.CRITICAL level."""
        return self._level_emit(self.logger.critical, metrics)

    def debug(self, **metrics):
        """Log the metrics expressed in keyword arguments at logging.DEBUG level."""
        return self._level_emit(self.logger.debug, metrics)

    def error(self, **metrics):
        """Log the metrics expressed in keyword arguments at logging.ERROR level."""
        return self._level_emit(self.logger.error, metrics)

    def fatal(self, **metrics):
        """Log the metrics expressed in keyword arguments at logging.CRITICAL level."""
        return self._level_emit(self.logger.fatal, metrics)

    def info(self, **metrics):
        """Log the metrics expressed in keyword arguments at logging.INFO level."""
        return self._level_emit(self.logger.info, metrics)

    def warn(self, **metrics):
        """Log the metrics expressed in keyword arguments at logging.WARN level."""
        return self._level_emit(self.logger.warn, metrics)

    def warning(self, **metrics):
        """Log the metrics expressed in keyword arguments at logging.WARN level."""
        return self._level_emit(self.logger.warning, metrics)


def to_metric_logger_name(logger_or_name):
    """Translates a `logger_or_name` to a standardized metrics-oriented logger name.

    If `logger_or_name` is a `logging.Logger`-like object, its `name` is used as the
    original name string; otherwise, `logger_or_name` is assumed to be a logger name
    string.

    The metrics logger name translation rules attempt to work well with the hierarchical
    naming of loggers enforced by the `logging` subsystem.  It will attempt to preserve
    the base package name of `logger_name` ("foo", in the case of a logger name "foo.blah").
    But it will attempt to put "metrics" into the name as the second level, such that it is
    easy within your application logging configuration to handle metrics as a separate stream
    from standard logs.

    Thus "foo" would go to "foo.metrics", while "foo.bar.baz" would go to "foo.metrics.bar.baz".

    If the logger name is the empty string, then it simply goes to "metrics".

    Parameters:
        logger_or_name: A standard name string for a logging.Logger, of the sort one
                        would use with logging.getLogger(), or a logging.Logger-like
                        object (the name of which is used as the name string).

    Returns the "metrics" name for logger_or_name, according to the rules stated above.
    """

    if hasattr(logger_or_name, 'name'):
        logger_or_name = logger_or_name.name

    if len(logger_or_name) > 0:
       idx = logger_or_name.find('.')
       if idx == -1:
           return "%s.metrics" % logger_or_name
       else:
           return "%s.metrics%s" % (logger_or_name[:idx], logger_or_name[idx:])
    else:
        return 'metrics'





from nose import tools
import mock
import six

import phlawg

NAME_MAP = {
    # Empty name goes to "metrics"
    '': 'metrics',
    # Single-level name gets "metrics" as sublevel
    'fooey': 'fooey.metrics',
    # Multi-level names get "metrics" as second level, regardless of depth
    'these.two': 'these.metrics.two',
    'this.has.three': 'this.metrics.has.three',
    'but.he.got.four': 'but.metrics.he.got.four',
}

class TestLoggerNames(object):
    def test_name_to_metric_name_translation(self):
        for original_name, expected_name in six.iteritems(NAME_MAP):
            yield self.verify_name_translation, original_name, expected_name

    def verify_name_translation(self, original, expected):
        tools.assert_equal(
            expected,
            phlawg.to_metric_logger_name(original))


    def test_logger_to_metric_name_translation(self):
        for original_name, expected_name in six.iteritems(NAME_MAP):
            yield self.verify_logger_translation, original_name, expected_name


    def verify_logger_translation(self, original, expected):
        logger = mock.Mock(name='Logger')
        logger.name = original
        tools.assert_equal(
            expected,
            phlawg.to_metric_logger_name(logger))




from setuptools import setup
import os

setup(
    name = "phlawg",
    version = "0.3.0",
    author = "Ethan Rowe",
    author_email = "ethan@the-rowes.com",
    description = "python logging utilities for transmitting metrics via log streams",
    license = "MIT",
    keywords = "metrics logging configuration setup",
    url = "https://github.com/ethanrowe/python-phlawg",
    packages=['phlawg',
              'phlawg.test',
    ],
    long_description="python logging utilities for transmitting metrics via log streams",
    test_suite='phlawg.test',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Topic :: Utilities",
    ],
    install_requires=[
        'python-json-logger>=0.1.4',
        'six>=1.9.0'
    ],
    tests_require=[
        'mock',
        'nose',
        'six>=1.9.0'
    ]
)

